@@ -44,9 +44,9 @@ class ServerCLITests(testing.TestCase):
             'os': 'Ubuntu',
             'os_version': 'Ubuntu 12.04 LTS',
             'owner': 'chechu',
-            'price_rate': 21.08,
+            'prices': [{'Item': 'Total', 'Recurring Price': 16.08},
+                       {'Item': 'test', 'Recurring Price': 1}],
             'private_ip': '10.1.0.2',
-            'ptr': '2.0.1.10.in-addr.arpa',
             'public_ip': '172.16.1.100',
             'remote users': [{'password': 'abc123', 'ipmi_username': 'root'}],
             'status': 'ACTIVE',
@@ -264,7 +264,9 @@ class ServerCLITests(testing.TestCase):
         expected = [
             [{'datacenter': 'Washington 1', 'value': 'wdc01'}],
             [{'size': 'Single Xeon 1270, 8GB Ram, 2x1TB SATA disks, Non-RAID',
-              'value': 'S1270_8GB_2X1TBSATA_NORAID'}],
+              'value': 'S1270_8GB_2X1TBSATA_NORAID'},
+             {'size': 'Dual Xeon Gold, 384GB Ram, 4x960GB SSD, RAID 10',
+              'value': 'DGOLD_6140_384GB_4X960GB_SSD_SED_RAID_10'}],
             [{'operating_system': 'Ubuntu / 14.04-64',
               'value': 'UBUNTU_14_64'}],
             [{'port_speed': '10 Mbps Public & Private Network Uplinks',
@@ -462,3 +464,50 @@ class ServerCLITests(testing.TestCase):
 
         self.assertEqual(result.exit_code, 2)
         self.assertIsInstance(result.exception, exceptions.CLIAbort)
+
+    def test_ready(self):
+        mock = self.set_mock('SoftLayer_Hardware_Server', 'getObject')
+        mock.return_value = {
+            "provisionDate": "2017-10-17T11:21:53-07:00",
+            "id": 41957081
+        }
+        result = self.run_command(['hw', 'ready', '100'])
+        self.assert_no_fail(result)
+        self.assertEqual(result.output, '"READY"\n')
+
+    def test_not_ready(self):
+        mock = self.set_mock('SoftLayer_Hardware_Server', 'getObject')
+        not_ready = {
+            'activeTransaction': {
+                'transactionStatus': {'friendlyName': 'Attach Primary Disk'}
+            },
+            'provisionDate': '',
+            'id': 47392219
+        }
+        ready = {
+            "provisionDate": "2017-10-17T11:21:53-07:00",
+            "id": 41957081
+        }
+        mock.side_effect = [not_ready, ready]
+        result = self.run_command(['hw', 'ready', '100'])
+        self.assertEqual(result.exit_code, 2)
+        self.assertIsInstance(result.exception, exceptions.CLIAbort)
+
+    @mock.patch('time.sleep')
+    def test_going_ready(self, _sleep):
+        mock = self.set_mock('SoftLayer_Hardware_Server', 'getObject')
+        not_ready = {
+            'activeTransaction': {
+                'transactionStatus': {'friendlyName': 'Attach Primary Disk'}
+            },
+            'provisionDate': '',
+            'id': 47392219
+        }
+        ready = {
+            "provisionDate": "2017-10-17T11:21:53-07:00",
+            "id": 41957081
+        }
+        mock.side_effect = [not_ready, ready]
+        result = self.run_command(['hw', 'ready', '100', '--wait=100'])
+        self.assert_no_fail(result)
+        self.assertEqual(result.output, '"READY"\n')
